require 'spec_helper'

describe Stacko::Builder do
  it 'has implicit #to_app' do
    app = builder do
      run lambda { |env| {:blah => 'blah'} }
    end

    app.call({}).must_equal({:blah=>"blah"})
  end

  it 'has explicit #to_app' do
    app = builder do
      run lambda { |env| {:blah => 'blah'} }
    end

    app.to_app.call({}).must_equal({:blah=>"blah"})
  end

  it 'chains app by default' do
    app = builder do
      use DogMiddleware
      run lambda { |env| env['ARGUMENTS'].push(:app); env['ARGUMENTS'] }
    end

    e = new_env(:blah, [])
    app.call(e).must_equal([:dog_middleware, :app])
  end

  it 'supports method mapping' do
    app = builder do
      map(:method_name_1) do
        run lambda { |env| 'method_name_1_was_called' }
      end

      map(:method_name_2) do
        run lambda { |env| 'method_name_2_was_called' }
      end
    end

    env = new_env(:method_name_1)
    app.call(env).must_equal('method_name_1_was_called')

    env = new_env(:method_name_2)
    app.call(env).must_equal('method_name_2_was_called')
  end

  it 'responds to call when mappings exist' do
    app = builder do
      map(:method_name_1) do
        run lambda { |env| 'method_name_1_was_called' }
      end

      run lambda { |env| 'default app called' }
    end

    env = new_env(:call)
    app.call(env).must_equal('default app called')
  end

  it 'correctly chains app in mappings' do
    app = builder do
      use BeaverMiddleware

      map(:method_name_1) do
        use DogMiddleware
        run lambda { |env| env['ARGUMENTS'].push(:meth1); env['ARGUMENTS'] }
      end

      map(:method_name_2) do
        use HorseMiddleware
        run lambda { |env| env['ARGUMENTS'].push(:meth2); env['ARGUMENTS'] }
      end

      run lambda { |env| env['ARGUMENTS'].push(:defaultapp); env['ARGUMENTS'] }
    end

    e = new_env(:call, [])
    app.call(e).must_equal([:beaver_middleware, :defaultapp])

    e = new_env(:method_name_1, [])
    app.call(e).must_equal([:beaver_middleware, :dog_middleware, :meth1])

    e = new_env(:method_name_2, [])
    app.call(e).must_equal([:beaver_middleware, :horse_middleware, :meth2])
  end

  it 'fails if run is missing' do
    app = builder do
      # no run statement
    end
    proc { app.call({}) } .must_raise(RuntimeError)
  end

  def builder(&block)
    Stacko::Builder.new(&block)
  end

  def new_env(method_name, arguments=nil)
    {
      'METHOD_NAME' => method_name,
      'ARGUMENTS' => arguments
    }
  end
end